@@ -1,15 +1,14 @@
-{-# OPTIONS_GHC -fno-warn-orphans #-}
-{-# LANGUAGE CPP #-}
-{-# LANGUAGE FlexibleInstances #-}
-{-# LANGUAGE LambdaCase #-}
+{-# OPTIONS_GHC -fno-warn-orphans  #-}
+{-# LANGUAGE CPP                   #-}
+{-# LANGUAGE FlexibleInstances     #-}
+{-# LANGUAGE LambdaCase            #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
-{-# LANGUAGE NamedFieldPuns #-}
-{-# LANGUAGE OverloadedStrings #-}
-{-# LANGUAGE QuasiQuotes #-}
-{-# LANGUAGE TemplateHaskell #-}
-{-# LANGUAGE TypeFamilies #-}
-{-# LANGUAGE TypeSynonymInstances #-}
-{-# LANGUAGE ViewPatterns #-}
+{-# LANGUAGE NamedFieldPuns        #-}
+{-# LANGUAGE OverloadedStrings     #-}
+{-# LANGUAGE QuasiQuotes           #-}
+{-# LANGUAGE TemplateHaskell       #-}
+{-# LANGUAGE TypeFamilies          #-}
+{-# LANGUAGE ViewPatterns          #-}
 
 -- | Define the web application's foundation, in the usual Yesod style.
 --   See a default Yesod app's comments for more details of each part.
@@ -22,16 +21,15 @@ import qualified Data.ByteString.Char8 as BC
 import Data.Traversable (for)
 import Data.IORef (IORef, readIORef, writeIORef)
 import Data.Maybe (fromMaybe)
-#if !(MIN_VERSION_base(4,13,0))
-import Data.Monoid ((<>))
-#endif
 import Data.Text (Text)
 import qualified Data.Text as T
 import Data.Time.Calendar (Day)
 import Network.HTTP.Conduit (Manager)
 import Network.HTTP.Types (status403)
 import Network.Wai (requestHeaders)
-import System.FilePath (takeFileName)
+import System.Directory (XdgDirectory (..), createDirectoryIfMissing,
+                         getXdgDirectory)
+import System.FilePath (takeFileName, (</>))
 import Text.Blaze (Markup)
 import Text.Hamlet (hamletFile)
 import Yesod
@@ -89,20 +87,18 @@ mkYesodData "App" $(parseRoutesFile "config/routes")
 -- | A convenience alias.
 type AppRoute = Route App
 
-#if MIN_VERSION_yesod(1,6,0)
 type Form x = Html -> MForm (HandlerFor App) (FormResult x, Widget)
-#else
-type Form x = Html -> MForm (HandlerT App IO) (FormResult x, Widget)
-#endif
 
 -- Please see the documentation for the Yesod typeclass. There are a number
 -- of settings which can be configured by overriding methods here.
 instance Yesod App where
   approot = ApprootMaster $ appRoot . settings
 
-  makeSessionBackend _ =
+  makeSessionBackend _ = do
+    hledgerdata <- getXdgDirectory XdgCache "hledger"
+    createDirectoryIfMissing True hledgerdata
     let sessionexpirysecs = 120
-    in  Just <$> defaultClientSessionBackend sessionexpirysecs ".hledger-web_client_session_key.aes"
+    Just <$> defaultClientSessionBackend sessionexpirysecs (hledgerdata </> "hledger-web_client_session_key.aes")
 
   -- defaultLayout :: WidgetFor site () -> HandlerFor site Html
   defaultLayout widget = do
@@ -117,16 +113,22 @@ instance Yesod App where
     VD {caps, j, m, opts, q, qopts} <- getViewData
     msg <- getMessage
     showSidebar <- shouldShowSidebar
-    hideEmptyAccts <- (== Just "1") . lookup "hideemptyaccts" . reqCookies <$> getRequest
 
-    let ropts = reportopts_ (cliopts_ opts)
-        ropts' = ropts
+    let rspec = reportspec_ (cliopts_ opts)
+        ropts = _rsReportOpts rspec
+        ropts' = (_rsReportOpts rspec)
           {accountlistmode_ = ALTree  -- force tree mode for sidebar
-          ,empty_           = not (empty_ ropts)  -- show zero items by default
+          ,empty_           = True    -- show zero items by default
           }
-        accounts =
+        rspec' = rspec{_rsQuery=m,_rsReportOpts=ropts'}
+
+    hideEmptyAccts <- if empty_ ropts
+                         then return True
+                         else (== Just "1") . lookup "hideemptyaccts" . reqCookies <$> getRequest
+
+    let accounts =
           balanceReportAsHtml (JournalR, RegisterR) here hideEmptyAccts j q qopts $
-          balanceReport ropts' m j
+          balanceReport rspec' j
 
         topShowmd = if showSidebar then "col-md-4" else "col-any-0" :: Text
         topShowsm = if showSidebar then "col-sm-4" else "" :: Text
@@ -154,6 +156,7 @@ instance Yesod App where
       addScript $ StaticR js_jquery_cookie_js
       addScript $ StaticR js_jquery_hotkeys_js
       addScript $ StaticR js_jquery_flot_min_js
+      addScript $ StaticR js_jquery_flot_selection_min_js
       addScript $ StaticR js_jquery_flot_time_min_js
       addScript $ StaticR js_jquery_flot_tooltip_min_js
       toWidget [hamlet| \<!--[if lte IE 8]> <script type="text/javascript" src="@{StaticR js_excanvas_min_js}"></script> <![endif]--> |]
@@ -199,15 +202,14 @@ instance Show Text.Blaze.Markup where show _ = "<blaze markup>"
 -- | Gather data used by handlers and templates in the current request.
 getViewData :: Handler ViewData
 getViewData = do
-  App {appOpts = opts, appJournal} <- getYesod
-  today <- liftIO getCurrentDay
-  let copts = cliopts_ opts
+  App{appOpts=opts@WebOpts{cliopts_=copts@CliOpts{reportspec_=rspec@ReportSpec{_rsReportOpts}}}, appJournal} <- getYesod
+  let today = _rsDay rspec
 
   -- try to read the latest journal content, keeping the old content
   -- if there's an error
   (j, mjerr) <- getCurrentJournal
                 appJournal
-                copts {reportopts_ = (reportopts_ copts) {no_elide_ = True}}
+                copts{reportspec_=rspec{_rsReportOpts=_rsReportOpts{no_elide_=True}}}
                 today
 
   -- try to parse the query param, assuming no query if there's an error
@@ -247,10 +249,7 @@ shouldShowSidebar = do
   msidebarcookie <- lookup "showsidebar" . reqCookies <$> getRequest
   return $
     let disablevalues = ["","0"]
-    in maybe
-         (not $ msidebarcookie `elem` map Just disablevalues)
-         (not . (`elem` disablevalues))
-         msidebarparam
+    in maybe True (`notElem` disablevalues) $ msidebarparam <|> msidebarcookie
 
 -- | Update our copy of the journal if the file changed. If there is an
 -- error while reloading, keep the old one and return the error, and set a
@@ -261,7 +260,7 @@ getCurrentJournal jref opts d = do
   j <- liftIO (readIORef jref)
   (ej, changed) <- liftIO $ journalReloadIfChanged opts d j
   -- re-apply any initial filter specified at startup
-  let initq = queryFromOpts d (reportopts_ opts)
+  let initq = _rsQuery $ reportspec_ opts
   case (changed, filterJournalTransactions initq <$> ej) of
     (False, _) -> return (j, Nothing)
     (True, Right j') -> do
