@@ -1,10 +1,11 @@
 -- | /register handlers.
 
-{-# LANGUAGE NamedFieldPuns #-}
-{-# LANGUAGE OverloadedStrings #-}
-{-# LANGUAGE QuasiQuotes #-}
+{-# LANGUAGE NamedFieldPuns      #-}
+{-# LANGUAGE OverloadedStrings   #-}
+{-# LANGUAGE QuasiQuotes         #-}
 {-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE TemplateHaskell #-}
+{-# LANGUAGE TemplateHaskell     #-}
+{-# LANGUAGE TupleSections       #-}
 
 module Hledger.Web.Handler.RegisterR where
 
@@ -19,7 +20,7 @@ import Hledger.Web.WebOptions
 import Hledger.Web.Widget.AddForm (addModal)
 import Hledger.Web.Widget.Common
              (accountQuery, mixedAmountAsHtml,
-              transactionFragment, removeInacct, replaceInacct)
+              transactionFragment, removeDates, removeInacct, replaceInacct)
 
 -- | The main journal/account register view, with accounts sidebar.
 getRegisterR :: Handler Html
@@ -33,7 +34,7 @@ getRegisterR = do
       s2 = if m /= Any then ", filtered" else ""
       header = a <> s1 <> s2
 
-  let ropts = reportopts_ (cliopts_ opts)
+  let rspec = reportspec_ (cliopts_ opts)
       acctQuery = fromMaybe Any (inAccountQuery qopts)
       acctlink acc = (RegisterR, [("q", replaceInacct q $ accountQuery acc)])
       otherTransAccounts =
@@ -44,8 +45,11 @@ getRegisterR = do
           zip xs $
           zip (map (T.unpack . accountSummarisedName . paccount) xs) $
           tail $ (", "<$xs) ++ [""]
-      r@(balancelabel,items) = accountTransactionsReport ropts j m acctQuery
-      balancelabel' = if isJust (inAccount qopts) then balancelabel else "Total"
+      items = accountTransactionsReport rspec{_rsQuery=m} j acctQuery
+      balancelabel
+        | isJust (inAccount qopts), balanceaccum_ (_rsReportOpts rspec) == Historical = "Historical Total"
+        | isJust (inAccount qopts) = "Period Total"
+        | otherwise                = "Total"
       transactionFrag = transactionFragment j
   defaultLayout $ do
     setTitle "register - hledger-web"
@@ -75,7 +79,7 @@ preferReal ps
 elideRightDecorated :: Int -> [(Maybe d, Char)] -> [(Maybe d, Char)]
 elideRightDecorated width s =
     if length s > width
-        then take (width - 2) s ++ map ((,) Nothing) ".."
+        then take (width - 2) s ++ map (Nothing,) ".."
         else s
 
 undecorateLinks :: [(Maybe acct, char)] -> [(acct, ([char], [char]))]
@@ -89,25 +93,23 @@ undecorateLinks xs0@(x:_) =
         _ -> error "link name not decorated with account"  -- PARTIAL:
 
 decorateLinks :: [(acct, ([char], [char]))] -> [(Maybe acct, char)]
-decorateLinks =
-    concatMap
-        (\(acct, (name, comma)) ->
-            map ((,) (Just acct)) name ++ map ((,) Nothing) comma)
+decorateLinks = concatMap $ \(acct, (name, comma)) ->
+    map (Just acct,) name ++ map (Nothing,) comma
 
 -- | Generate javascript/html for a register balance line chart based on
--- the provided "TransactionsReportItem"s.
-registerChartHtml :: [(CommoditySymbol, (String, [TransactionsReportItem]))] -> HtmlUrl AppRoute
-registerChartHtml percommoditytxnreports = $(hamletFile "templates/chart.hamlet")
+-- the provided "AccountTransactionsReportItem"s.
+registerChartHtml :: Text -> String -> [(CommoditySymbol, [AccountTransactionsReportItem])] -> HtmlUrl AppRoute
+registerChartHtml q title percommoditytxnreports = $(hamletFile "templates/chart.hamlet")
  -- have to make sure plot is not called when our container (maincontent)
  -- is hidden, eg with add form toggled
  where
-   charttitle = case maybe "" (fst . snd) $ listToMaybe percommoditytxnreports of
-     "" -> ""
-     s  -> s <> ":"
+   charttitle = if null title then "" else title ++ ":"
    colorForCommodity = fromMaybe 0 . flip lookup commoditiesIndex
    commoditiesIndex = zip (map fst percommoditytxnreports) [0..] :: [(CommoditySymbol,Int)]
-   simpleMixedAmountQuantity = maybe 0 aquantity . listToMaybe . amounts
+   simpleMixedAmountQuantity = maybe 0 aquantity . listToMaybe . amounts . mixedAmountStripPrices
+   showZeroCommodity = wbUnpack . showMixedAmountB oneLine{displayPrice=False,displayZeroCommodity=True}
    shownull c = if null c then " " else c
+   nodatelink = (RegisterR, [("q", T.unwords $ removeDates q)])
 
 dayToJsTimestamp :: Day -> Integer
 dayToJsTimestamp d =
