@@ -1,4 +1,3 @@
-{-# LANGUAGE CPP #-}
 module Hledger.Web.Import
   ( module Import
   ) where
@@ -25,7 +24,3 @@ import           Hledger.Web.Foundation           as Import
 import           Hledger.Web.Settings             as Import
 import           Hledger.Web.Settings.StaticFiles as Import
 import           Hledger.Web.WebOptions           as Import (Capability(..))
-
-#if !(MIN_VERSION_base(4,11,0))
-import           Data.Monoid          as Import ((<>))
-#endif
