@@ -1,12 +1,12 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE OverloadedStrings #-}
+
 module Hledger.Web.WebOptions where
 
 import Data.ByteString (ByteString)
 import qualified Data.ByteString.Char8 as BC
 import Data.ByteString.UTF8 (fromString)
 import Data.CaseInsensitive (CI, mk)
-import Control.Monad (join)
 import Data.Default (Default(def))
 import Data.Maybe (fromMaybe)
 import qualified Data.Text as T
@@ -15,18 +15,25 @@ import System.Environment (getArgs)
 import Network.Wai as WAI
 import Network.Wai.Middleware.Cors
 
-import Hledger.Cli hiding (progname, version)
+import Hledger.Cli hiding (packageversion, progname, prognameandversion)
 import Hledger.Web.Settings (defhost, defport, defbaseurl)
 
-progname, version :: String
-progname = "hledger-web"
+-- cf Hledger.Cli.Version
+
+packageversion :: PackageVersion
+packageversion =
 #ifdef VERSION
-version = VERSION
+  VERSION
 #else
-version = ""
+  ""
 #endif
-prognameandversion :: String
-prognameandversion = progname ++ " " ++ version :: String
+
+progname :: ProgramName
+progname = "hledger-web"
+
+prognameandversion :: VersionString
+prognameandversion = versionString progname packageversion
+
 
 webflags :: [Flag RawOpts]
 webflags =
@@ -78,6 +85,10 @@ webflags =
       (\s opts -> Right $ setopt "capabilities-header" s opts)
       "HTTPHEADER"
       "read capabilities to enable from a HTTP header, like X-Sandstorm-Permissions (default: disabled)"
+  , flagNone
+      ["test"]
+      (setboolopt "test")
+      "run hledger-web's tests and exit. hspec test runner args may follow a --, eg: hledger-web --test -- --help"
   ]
 
 webmode :: Mode RawOpts
@@ -92,12 +103,13 @@ webmode =
       Group
       { groupUnnamed = webflags
       , groupHidden =
-          hiddenflags ++
-          [ flagNone
-              ["binary-filename"]
-              (setboolopt "binary-filename")
-              "show the download filename for this executable, and exit"
-          ]
+          hiddenflags 
+          -- ++
+          -- [ flagNone
+          --     ["binary-filename"]
+          --     (setboolopt "binary-filename")
+          --     "show the download filename for this executable, and exit"
+          -- ]
       , groupNamed = [generalflagsgroup1]
       }
   , modeHelpSuffix = []
@@ -119,7 +131,19 @@ data WebOpts = WebOpts
   } deriving (Show)
 
 defwebopts :: WebOpts
-defwebopts = WebOpts def def Nothing def def def def [CapView, CapAdd] Nothing def Nothing
+defwebopts = WebOpts
+  { serve_              = False
+  , serve_api_          = False
+  , cors_               = Nothing
+  , host_               = ""
+  , port_               = def
+  , base_url_           = ""
+  , file_url_           = Nothing
+  , capabilities_       = [CapView, CapAdd]
+  , capabilitiesHeader_ = Nothing
+  , cliopts_            = def
+  , socket_             = Nothing
+  }
 
 instance Default WebOpts where def = defwebopts
 
@@ -132,7 +156,7 @@ rawOptsToWebOpts rawopts =
         b =
           maybe (defbaseurl h p) stripTrailingSlash $
           maybestringopt "base-url" rawopts
-        caps' = join $ T.splitOn "," . T.pack <$> listofstringopt "capabilities" rawopts
+        caps' = T.splitOn "," . T.pack =<< listofstringopt "capabilities" rawopts
         caps = case traverse capabilityFromText caps' of
           Left e -> error' ("Unknown capability: " ++ T.unpack e)  -- PARTIAL:
           Right [] -> [CapView, CapAdd]
