@@ -1,9 +1,8 @@
-{-# LANGUAGE CPP #-}
-{-# LANGUAGE LambdaCase #-}
-{-# LANGUAGE NamedFieldPuns #-}
+{-# LANGUAGE LambdaCase        #-}
+{-# LANGUAGE NamedFieldPuns    #-}
 {-# LANGUAGE OverloadedStrings #-}
-{-# LANGUAGE QuasiQuotes #-}
-{-# LANGUAGE TemplateHaskell #-}
+{-# LANGUAGE QuasiQuotes       #-}
+{-# LANGUAGE TemplateHaskell   #-}
 
 module Hledger.Web.Widget.Common
   ( accountQuery
@@ -15,18 +14,15 @@ module Hledger.Web.Widget.Common
   , writeJournalTextIfValidAndChanged
   , journalFile404
   , transactionFragment
+  , removeDates
   , removeInacct
   , replaceInacct
   ) where
 
-import Data.Default (def)
 import Data.Foldable (find, for_)
-#if !(MIN_VERSION_base(4,13,0))
-import Data.Semigroup ((<>))
-#endif
+import Data.List (elemIndex)
 import Data.Text (Text)
 import qualified Data.Text as T
-import qualified Data.HashMap.Strict as HashMap
 import System.FilePath (takeFileName)
 import Text.Blaze ((!), textValue)
 import qualified Text.Blaze.Html5 as H
@@ -41,11 +37,7 @@ import Hledger.Cli.Utils (writeFileWithBackupIfChanged)
 import Hledger.Web.Settings (manualurl)
 import qualified Hledger.Query as Query
 
-#if MIN_VERSION_yesod(1,6,0)
 journalFile404 :: FilePath -> Journal -> HandlerFor m (FilePath, Text)
-#else
-journalFile404 :: FilePath -> Journal -> HandlerT m IO (FilePath, Text)
-#endif
 journalFile404 f j =
   case find ((== f) . fst) (jfiles j) of
     Just (_, txt) -> pure (takeFileName f, txt)
@@ -73,7 +65,7 @@ writeJournalTextIfValidAndChanged f t = do
   -- formatdirectivep, #1194) writeFileWithBackupIfChanged require them.
   -- XXX klunky. Any equivalent of "hSetNewlineMode h universalNewlineMode" for form posts ?
   let t' = T.replace "\r" "" t
-  liftIO (readJournal def (Just f) t') >>= \case
+  liftIO (readJournal definputopts (Just f) t') >>= \case
     Left e -> return (Left e)
     Right _ -> do
       _ <- liftIO (writeFileWithBackupIfChanged f t')
@@ -92,6 +84,8 @@ balanceReportAsHtml (journalR, registerR) here hideEmpty j q qopts (items, total
     l = ledgerFromJournal Any j
     indent a = preEscapedString $ concat $ replicate (2 + 2 * a) "&nbsp;"
     hasSubAccounts acct = maybe True (not . null . asubs) (ledgerAccount l acct)
+    isInterestingAccount acct = maybe False isInteresting $ ledgerAccount l acct
+      where isInteresting a = not (mixedAmountLooksZero (aebalance a)) || any isInteresting (asubs a)
     matchesAcctSelector acct = Just True == ((`matchesAccount` acct) <$> inAccountQuery qopts)
 
 accountQuery :: AccountName -> Text
@@ -110,12 +104,22 @@ mixedAmountAsHtml b _ =
       Just True -> "negative amount"
       _ -> "positive amount"
 
+-- Make a slug to uniquely identify this transaction
+-- in hyperlinks (as far as possible).
 transactionFragment :: Journal -> Transaction -> String
-transactionFragment j =
-    let hm = HashMap.fromList $ zip (map fst $ jfiles j) [(1::Integer) ..]
-    in  \t ->
-            printf "transaction-%d-%d"
-                (hm HashMap.! sourceFilePath (tsourcepos t)) (tindex t)
+transactionFragment j Transaction{tindex, tsourcepos} = 
+  printf "transaction-%d-%d" tfileindex tindex
+  where
+    -- the numeric index of this txn's file within all the journal files,
+    -- or 0 if this txn has no known file (eg a forecasted txn)
+    tfileindex = maybe 0 (+1) $ elemIndex (sourceName $ fst tsourcepos) (journalFilePaths j)
+
+removeDates :: Text -> [Text]
+removeDates =
+    map quoteIfSpaced .
+    filter (\term ->
+        not $ T.isPrefixOf "date:" term || T.isPrefixOf "date2:" term) .
+    Query.words'' Query.prefixes
 
 removeInacct :: Text -> [Text]
 removeInacct =
